using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public enum UploadFileType
    {
        Any,
        Image,
        Csv
    }

    public static class FileUtilities
    {
        // This function returns false if an error occurs. It does not true exception
        public static bool ContainsUnicodeBom( string filePath )
        {
            bool unicode = false;
            FileStream file = null;
            try
            {
                file = new FileStream( filePath, FileMode.Open, FileAccess.Read, FileShare.Read );

                if (file.CanSeek)
                {
                    byte[] bom = new byte[4]; // Get the byte-order mark, if there is one
                    file.Read( bom, 0, 4 );
                    if ((bom[0] == 0xef && bom[1] == 0xbb && bom[2] == 0xbf) || // utf-8
                        (bom[0] == 0xff && bom[1] == 0xfe) || // ucs-2le, ucs-4le, and ucs-16le
                        (bom[0] == 0xfe && bom[1] == 0xff) || // utf-16 and ucs-2
                        (bom[0] == 0 && bom[1] == 0 && bom[2] == 0xfe && bom[3] == 0xff)) // ucs-4
                    {
                        unicode = true;
                    }
                }
            }
            finally
            {
                if (file != null)
                    file.Close();
            }

            return unicode;
        }

        public static void SaveFile( string filePath, string data )
        {
            StreamWriter stream = File.CreateText( filePath );
            try
            {
                stream.Write( data );
            }
            finally
            {
                stream.Flush();
                stream.Close();
            }
        }

        public static bool CheckFileType( string filePath, UploadFileType checkType )
        {
            string fileExtension = Path.GetExtension( filePath ).ToLower();
            switch (checkType)
            {
                case UploadFileType.Image:
                    if (fileExtension == ".jpg" || 
                        fileExtension == ".jpeg" || 
                        fileExtension == ".gif" || 
                        fileExtension == ".png" || 
                        fileExtension == ".bmp" || 
                        fileExtension == ".tiff")
                        return true;
                    else
                        return false;
                case UploadFileType.Csv:
                    if (fileExtension == ".csv")
                        return true;
                    else
                        return false;
                default :
                    return true;
            }
        }

        public static bool Delete( string filePath )
        {
            if (String.IsNullOrEmpty( filePath ))
                return false;

            if (!File.Exists( filePath ))
                return false;

            try
            {
                File.Delete( filePath );
                return true;
            }
            catch
            {
                return false;
            }
        }

    }
}
