using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Contents;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentMenuRepositoryNoCache : IContentMenuRepository
    {
        #region Private
        private const string AllContentMenuFields = "ContentMenuID, IsEnabled ";

        private ContentMenu Create( ContentMenu contentmenu )
        {
            string newID = DataAccess.ExecuteScalar(
                    "INSERT INTO ContentMenu (IsEnabled)" +
                     "VALUES (@IsEnabled) ;" +
                     "SELECT @@Identity; " ,
                     DataAccess.CreateParameterBool(contentmenu.IsEnabled)
                     );
            contentmenu.ContentMenuID = newID;
            return contentmenu;
        }

        private ContentMenu Update( ContentMenu contentmenu )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ContentMenu " +
                "SET IsEnabled = @IsEnabled " +
                "WHERE ContentMenuID = @ContentMenuID;",
                DataAccess.CreateParameterBool(contentmenu.IsEnabled),
                DataAccess.CreateParameterString( contentmenu.ContentMenuID ) );

            return contentmenu;
        }

        private ContentMenu CreateOneFromRow( DataRow row )
        {
            ContentMenu contentmenu = new ContentMenu();
            contentmenu.ContentMenuID = row["ContentMenuID"].ToString();
            contentmenu.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );

            return contentmenu;
        }

        private IList<ContentMenu> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<ContentMenu> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<ContentMenu> list = new List<ContentMenu>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion

        #region Public Methods

        public ContentMenu Save( ContentMenu contentmenu )
        {
            ContentMenu result;

            if (contentmenu.ContentMenuID == "0")
                result = Create( contentmenu );
            else
                result = Update( contentmenu );

            return result;
        }

        public ContentMenu GetOne( string contentmenuID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllContentMenuFields +
                "FROM ContentMenu " +
                "WHERE ContentMenuID = @ContentMenuID; ",
                DataAccess.CreateParameterString( contentmenuID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return ContentMenu.Null;
        }


        public void Delete( string contentmenuID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ContentMenu " +
                "WHERE ContentMenuID = @ContentMenuID; ",
                DataAccess.CreateParameterString( contentmenuID ) );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllContentMenuFields +
                "FROM ContentMenu " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

      
        #endregion

    }
}
