using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class GiftRegistryItemRepositoryNoCache : IGiftRegistryItemRepository
    {
        #region Private

        private const string allFields = "GiftRegistryItemID, GiftRegistryID, ProductID, Productname, OptionItems," +
            " GiftCertificatePrice, WantQuantity, HasQuantity, IsGiftCertificate, Recipient, PersonalNote, NeedPhysical ";

        GiftRegistryItem Create( GiftRegistryItem giftRegistryItem )
        {
            string giftRegistryItemID = DataAccess.ExecuteScalar(
                "INSERT INTO GiftRegistryItem " +
                "(GiftRegistryID, ProductID, ProductName, OptionItems, GiftCertificatePrice, WantQuantity, HasQuantity, " +
                "IsGiftCertificate, Recipient, PersonalNote, NeedPhysical ) " +
                "VALUES (@GiftRegistryID, @ProductID, @ProductName, @OptionItems, @GiftCertificatePrice, @WantQuantity," +
                " @HasQuantity, @IsGiftCertificate, @Recipient, @PersonalNote, @NeedPhysical); " +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( giftRegistryItem.GiftRegistryID ),
                DataAccess.CreateParameterString( giftRegistryItem.ProductID ),
                DataAccess.CreateParameterString( giftRegistryItem.ProductName ),
                DataAccess.CreateParameterString( giftRegistryItem.OptionItems ),
                DataAccess.CreateParameterDecimal( giftRegistryItem.GiftCertificatePrice ),
                DataAccess.CreateParameterDecimal( giftRegistryItem.WantQuantity ),
                DataAccess.CreateParameterDecimal( giftRegistryItem.HasQuantity ),
                DataAccess.CreateParameterBool( giftRegistryItem.IsGiftCertificate ),
                DataAccess.CreateParameterString( giftRegistryItem.Recipient ),
                DataAccess.CreateParameterString( giftRegistryItem.PersonalNote ),
                DataAccess.CreateParameterBool( giftRegistryItem.NeedPhysical )
            );

            giftRegistryItem.GiftRegistryItemID = giftRegistryItemID;
            return giftRegistryItem;
        }

        private GiftRegistryItem Update( GiftRegistryItem giftRegistryItem )
        {
            DataAccess.ExecuteNonQuery(
                "Update GiftRegistryItem SET " +
                " WantQuantity = @WantQuantity, " +
                " HasQuantity = @HasQuantity " +
                "WHERE GiftRegistryItemID = @GiftRegistryItemID;",
                DataAccess.CreateParameterDecimal( giftRegistryItem.WantQuantity ),
                DataAccess.CreateParameterDecimal( giftRegistryItem.HasQuantity ),
                DataAccess.CreateParameterString( giftRegistryItem.GiftRegistryItemID )
            );

            return giftRegistryItem;
        }

        private GiftRegistryItem CreateOneFromRow( DataRow row )
        {
            GiftRegistryItem giftRegistryItem = new GiftRegistryItem();

            giftRegistryItem.GiftRegistryItemID = row["GiftRegistryItemID"].ToString();
            giftRegistryItem.GiftRegistryID = row["GiftRegistryID"].ToString();
            giftRegistryItem.ProductID = row["ProductID"].ToString();
            giftRegistryItem.ProductName = row["ProductName"].ToString();
            giftRegistryItem.OptionItems = row["OptionItems"].ToString();
            giftRegistryItem.GiftCertificatePrice = ConvertUtilities.ToDecimal( row["GiftCertificatePrice"] );
            giftRegistryItem.WantQuantity = ConvertUtilities.ToDecimal( row["WantQuantity"] );
            giftRegistryItem.HasQuantity = ConvertUtilities.ToDecimal( row["HasQuantity"] );
            giftRegistryItem.IsGiftCertificate = ConvertUtilities.ToBoolean( row["IsGiftCertificate"] );
            giftRegistryItem.Recipient = row["Recipient"].ToString();
            giftRegistryItem.PersonalNote = row["PersonalNote"].ToString();
            giftRegistryItem.NeedPhysical = ConvertUtilities.ToBoolean( row["NeedPhysical"] );

            return giftRegistryItem;
        }

        private IList<GiftRegistryItem> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<GiftRegistryItem> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GiftRegistryItem> list = new List<GiftRegistryItem>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public GiftRegistryItem Save( GiftRegistryItem giftRegistryItem )
        {
            GiftRegistryItem result;

            if (giftRegistryItem.GiftRegistryItemID == "0")
                result = Create( giftRegistryItem );
            else
                result = Update( giftRegistryItem );

            return result;
        }

        public GiftRegistryItem GetOne( string giftRegistryItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM GiftRegistryItem " +
                "WHERE GiftRegistryItemID = @GiftRegistryItemID;",
                DataAccess.CreateParameterString( giftRegistryItemID )
            );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GiftRegistryItem.Null;
        }

        public void Delete( string giftRegistryItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftRegistryItem " +
                "WHERE GiftRegistryItemID = @GiftRegistryItemID;",
                DataAccess.CreateParameterString( giftRegistryItemID )
            );
        }

        public void DeleteByGiftRegistryID( string giftRegistryID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftRegistryItem " +
                "WHERE GiftRegistryID = @GiftRegistryID;",
                DataAccess.CreateParameterString( giftRegistryID )
            );
        }

        public IList<GiftRegistryItem> GetAllByGiftRegistryID( string giftRegistryID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM GiftRegistryItem " +
                "WHERE GiftRegistryID = @GiftRegistryID;",
                DataAccess.CreateParameterString( giftRegistryID )
            );
            return ConvertTableToList( table );
        }

        public bool CanBeDeletedByGiftRegistryID( string giftRegistryID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM GiftRegistryItem " +
                "WHERE GiftRegistryID = @GiftRegistryID AND HasQuantity > 0;",
                DataAccess.CreateParameterString( giftRegistryID )
            );
            if (table.Rows.Count == 0)
                return true;
            else
                return false;
        }

        public bool CanBeDeletedByGiftRegistryItemID( string giftRegistryItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
             "SELECT HasQuantity " +
             "FROM GiftRegistryItem " +
             "WHERE GiftRegistryItemID = @GiftRegistryItemID;",
             DataAccess.CreateParameterString( giftRegistryItemID )
            );
            if (ConvertUtilities.ToDecimal( table.Rows[0]["HasQuantity"] ) == 0)
                return true;
            else
                return false;
        }

        public string GetGiftRegistryItemID(
            string giftRegistryID, 
            string productID,
            string optionItems,
            decimal giftCertificatePrice,
            bool isGiftCertificate,
            string recipient,
            string personalNote,
            bool needPhysical )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM GiftRegistryItem " +
                "WHERE GiftRegistryID = @GiftRegistryID " +
                "AND ProductID = @ProductID " +
                "AND OptionItems = @OptionItems " +
                "AND GiftCertificatePrice = @GiftCertificatePrice " +
                "AND IsGiftCertificate = @IsGiftCertificate " +
                "AND Recipient = @Recipient " +
                "AND PersonalNote = @PersonalNote " +
                "AND NeedPhysical = @NeedPhysical;",
                DataAccess.CreateParameterString( giftRegistryID ),
                DataAccess.CreateParameterString( productID ),
                DataAccess.CreateParameterString( optionItems ),
                DataAccess.CreateParameterDecimal( giftCertificatePrice ),
                DataAccess.CreateParameterBool( isGiftCertificate ),
                DataAccess.CreateParameterString( recipient ),
                DataAccess.CreateParameterString( personalNote ),
                DataAccess.CreateParameterBool( needPhysical )
            );

            if (table.Rows.Count == 0)
                return "0";
            else
                return table.Rows[0]["GiftREgistryItemID"].ToString();
        }

        #endregion
    }
}
