﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.GoogleFeed;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GoogleFeedTagValueRepository : IGoogleFeedTagValueRepository
    {
        private const string RepositoryName = "GoogleFeedTagValueRepository";

        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetOneSignature = "GetOne";
        private const string GetOneByTagIDSignature = "GetOneByTagID";
        private const string GetAllSignature = "GetAll";

        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "GoogleFeedTag", "GoogleFeedTagValue" } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private GoogleFeedTagValueRepositoryNoCache _repository = new GoogleFeedTagValueRepositoryNoCache();

        #region Constructors

        public GoogleFeedTagValueRepository()
        {
        }

        public GoogleFeedTagValueRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region GoogleFeedTagRepository Members

        public GoogleFeedTagValue Save( GoogleFeedTagValue googleFeedTagValue )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            GoogleFeedTagValue newGoogleFeedTagValue = _repository.Save( googleFeedTagValue );
            _dataCache.ClearRepositoryCache();
            return newGoogleFeedTagValue;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public GoogleFeedTagValue GetOne( string googleFeedTagValueID )
        {
            GoogleFeedTagValue result = (GoogleFeedTagValue) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, googleFeedTagValueID );

            if (result == null)
            {
                result = new GoogleFeedTagValue();
                result = _repository.GetOne( googleFeedTagValueID );
                _dataCache.SetValue(
                    GetOneSignature, 
                    result, 
                    googleFeedTagValueID 
                    );
            }
            return result;
        }

        public IList<GoogleFeedTagValue> GetAll()
        {
            IList<GoogleFeedTagValue> result = (IList<GoogleFeedTagValue>) _dataCache.GetValue(
                 GetAllSignature, _cacheDependency );

            if (result == null)
            {
                result = new List<GoogleFeedTagValue>();
                result = _repository.GetAll();
                _dataCache.SetValue(
                    GetAllSignature,
                    result
                    );
            }
            return result;
        }

        public IList<GoogleFeedTagValue> GetAll( string sortBy )
        {
            IList<GoogleFeedTagValue> result = (IList<GoogleFeedTagValue>) _dataCache.GetValue(
                 GetAllSignature, _cacheDependency, sortBy );

            if (result == null)
            {
                result = new List<GoogleFeedTagValue>();
                result = _repository.GetAll( sortBy );
                _dataCache.SetValue(
                    GetAllSignature,
                    result,
                    sortBy
                    );
            }
            return result;
        }

        public IList<GoogleFeedTagValue> GetAllByTagID( string googleFeedTagID )
        {
            IList<GoogleFeedTagValue> result = (IList<GoogleFeedTagValue>) _dataCache.GetValue(
                GetOneByTagIDSignature, _cacheDependency, googleFeedTagID );

            if (result == null)
            {
                result = _repository.GetAllByTagID( googleFeedTagID );
                _dataCache.SetValue( GetOneByTagIDSignature, result, googleFeedTagID );
            }

            return result;
        }

        public IList<GoogleFeedTagValue> GetAllByTagID( string sortBy, string googleFeedTagID )
        {
            IList<GoogleFeedTagValue> result = (IList<GoogleFeedTagValue>) _dataCache.GetValue(
                GetOneByTagIDSignature, _cacheDependency, sortBy, googleFeedTagID );

            if (result == null)
            {
                result = _repository.GetAllByTagID( googleFeedTagID );
                _dataCache.SetValue( GetOneByTagIDSignature, result, sortBy, googleFeedTagID );
            }

            return result;
        }

        public void Delete( string googleFeedTagValueID )
        {
            _repository.Delete( googleFeedTagValueID );
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
