using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;

namespace Vevo.Data.Implementation
{
    internal class ProductRecurringAccess
    {
        #region Private



        #endregion


        #region Public Methods

        public Product Create( Product product )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO ProductRecurring (ProductID, RecurringInterval, RecurringIntervalUnit, " +
                "RecurringNumberOfCycles, RecurringNumberOfTrialCycles, RecurringTrialAmount ) " +
                "VALUES (@ProductID, @Interval, @IntervalUnit, @IntervalNumberOfCycle, " +
                "@TrialIntervalNumberOfCycle, @TrialAmount );",
                DataAccess.CreateParameterString( product.ProductID ),
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringInterval ),
                DataAccess.CreateParameterString( product.ProductRecurring.RecurringIntervalUnit ),
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfCycles ),
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfTrialCycles ),
                DataAccess.CreateParameterDecimalMoney( product.ProductRecurring.RecurringTrialAmount )
                );

            return product;
        }

        public Product Update( Product product )
        {
            int update = DataAccess.ExecuteNonQuery(
                "UPDATE ProductRecurring " +
                "SET RecurringInterval = @RecurringInterval, " +
                "    RecurringIntervalUnit = @RecurringIntervalUnit, " +
                "    RecurringNumberOfCycles = @RecurringNumberOfCycles, " +
                "    RecurringNumberOfTrialCycles = @RecurringIntervalNumberOfTrialCycle, " +
                "    RecurringTrialAmount = @RecurringTrialAmount " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringInterval ),
                DataAccess.CreateParameterString( product.ProductRecurring.RecurringIntervalUnit ),
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfCycles ),
                DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfTrialCycles ),
                DataAccess.CreateParameterDecimalMoney( product.ProductRecurring.RecurringTrialAmount ),
                DataAccess.CreateParameterString( product.ProductID ) );

            // Recurring data may not exist yet
            // Insert a new data if necessary
            if (update == 0)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO ProductRecurring (ProductID, RecurringInterval, RecurringIntervalUnit, " +
                    "    RecurringNumberOfCycles, RecurringNumberOfTrialCycles, RecurringTrialAmount ) " +
                    "VALUES (@ProductID, @Interval, @IntervalUnit, @IntervalNumberOfCycle, " +
                    "    @TrialIntervalNumberOfCycle, @TrialAmount);",
                    DataAccess.CreateParameterString( product.ProductID ),
                    DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringInterval ),
                    DataAccess.CreateParameterString( product.ProductRecurring.RecurringIntervalUnit ),
                    DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfCycles ),
                    DataAccess.CreateParameterInt32( product.ProductRecurring.RecurringNumberOfTrialCycles ),
                    DataAccess.CreateParameterDecimalMoney( product.ProductRecurring.RecurringTrialAmount ) );
            }

            return product;
        }

        #endregion

    }
}
