using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain.Payments;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Shared.DataAccess;

namespace Vevo.Data
{
    public class PaymentLogRepositoryNoCache : IPaymentLogRepository
    {
        #region Private

        private const string AllPaymentLogFields =
            "PaymentLogID, OrderID, TimeStamp, PaymentResponse, PaymentGateWay, PaymentType ";

        private PaymentLog Create( PaymentLog paymentLog )
        {
            string newPaymentLogID = DataAccess.ExecuteScalar(
                "INSERT INTO PaymentLog ( OrderID, [TimeStamp], PaymentResponse, PaymentGateWay, PaymentType ) " +
                "VALUES ( @OrderID, @TimeStamp, @PaymentResponse, @PaymentGateWay, @PaymentType );" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( paymentLog.OrderID ),
                DataAccess.CreateParameterDateTime( paymentLog.TimeStamp ),
                DataAccess.CreateParameterString( paymentLog.PaymentResponse ),
                DataAccess.CreateParameterString( paymentLog.PaymentGateway ),
                DataAccess.CreateParameterString( paymentLog.PaymentType ) );

            paymentLog.PaymentLogID = newPaymentLogID;
            return paymentLog;
        }

        private PaymentLog CreateOneFromRow( DataRow row )
        {
            PaymentLog paymentLog = new PaymentLog();
            paymentLog.PaymentLogID = row["PaymentLogID"].ToString();
            paymentLog.OrderID = row["OrderID"].ToString();
            paymentLog.TimeStamp = ConvertUtilities.ToDateTime( row["TimeStamp"] );
            paymentLog.PaymentResponse = row["PaymentResponse"].ToString();
            paymentLog.PaymentGateway = row["PaymentGateway"].ToString();
            paymentLog.PaymentType = row["PaymentType"].ToString();

            return paymentLog;
        }

        #endregion

        #region Public

        public PaymentLog Save( PaymentLog paymentLog )
        {
            return Create( paymentLog );
        }

        public PaymentLog GetOne( string paymentLogID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllPaymentLogFields +
                "FROM PaymentLog " +
                "WHERE PaymentLogID = @PaymentLogID; ",
                DataAccess.CreateParameterString( paymentLogID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return PaymentLog.Null;
        }

        #endregion
    }
}
