﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.ImportExport
{
    public class CustomerImportStatus
    {
        public enum ErrorStatus { Success, DoneWithErrors, Error, Fatal }

        public ErrorStatus Status
        {
            get
            {
                return _status;
            }
        }

        public int ImportedCount;
        public int UpdatedCount;
        public int ErrorCount;
        public int SkippedCount;
        public string SkippedCustomerMessage;
        public ArrayList SkippedList;
        public ArrayList OtherErrorList;
        public ArrayList OtherErrorExceptionList;
        public string OtherError;
        public string CustomerNotFoundError;
        public ArrayList CustomerNotFoundErrorList;
        public string HeaderError;
        public ArrayList HeaderErrorList;
        public string FieldError;
        public ArrayList FieldErrorList;
        public string ShippingFieldError;
        public ArrayList ShippingFieldErrorList;

        public string ErrorMessage
        {
            get
            {
                if (ErrorCount > 0 || _status == ErrorStatus.Fatal)
                    return _errorBuilder.ToString();
                else
                    return String.Empty;
            }
        }

        public CustomerImportStatus()
        {
            ImportedCount = 0;
            UpdatedCount = 0;
            ErrorCount = 0;
            SkippedCount = 0;
            SkippedCustomerMessage = String.Empty;
            SkippedList = new ArrayList();
            OtherErrorList = new ArrayList();
            OtherErrorExceptionList = new ArrayList();
            OtherError = String.Empty;
            CustomerNotFoundError = String.Empty;
            CustomerNotFoundErrorList = new ArrayList();
            HeaderError = String.Empty;
            HeaderErrorList = new ArrayList();
            FieldError = String.Empty;
            FieldErrorList = new ArrayList();
            ShippingFieldError = String.Empty;
            ShippingFieldErrorList = new ArrayList();
        }

        private StringBuilder _errorBuilder = new StringBuilder();
        private ErrorStatus _status = ErrorStatus.Success;

        public void AddSuccess()
        {
            ImportedCount++;
        }

        public void UpdateSuccess()
        {
            UpdatedCount++;
        }

        public void Skip()
        {
            SkippedCount++;
        }

        public void AddError()
        {
            ErrorCount++;
        }

        public void AddOtherError( string lineNumber, Exception ex )
        {
            OtherErrorList.Add( lineNumber );
            OtherErrorExceptionList.Add( ex );

            _status = ErrorStatus.DoneWithErrors;

            AddError();
        }

        public void AddSkippedCustomer( string lineNumber )
        {
            SkippedList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
        }

        public void AddCustomerNotFoundError( string lineNumber )
        {
            CustomerNotFoundErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            AddError();
        }

        public void SetFatalError( string message )
        {
            _errorBuilder = new StringBuilder( message );
            _status = ErrorStatus.Fatal;
        }

        public void AddHeaderError(string headerName)
        {
            HeaderErrorList.Add( headerName );
            _status = ErrorStatus.Error;
            ErrorCount = 1;
        }

        public void AddFieldError( string lineNumber )
        {
            FieldErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            ErrorCount = 1;
        }

        public void AddShippingFieldError( string lineNumber )
        {
            ShippingFieldErrorList.Add( lineNumber );
            _status = ErrorStatus.DoneWithErrors;
            ErrorCount = 1;
        }

        private string CreateOtherErrorDetails()
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < 5 && i < OtherErrorExceptionList.Count; i++)
            {
                if (i > 0)
                    sb.Append( "<br/>----------<br/>" );

                Exception ex = (Exception) OtherErrorExceptionList[i];

                sb.Append( "<br/>Message: " );
                sb.Append( ex.Message );
                sb.Append( "<br/>Exception Type: " );
                sb.Append( ex.GetType().ToString() );
                sb.Append( "<br/>Stack Trace:<br/>" );
                sb.Append( HtmlFormatter.ConvertNewLines( ex.StackTrace ) );
            }

            return sb.ToString();
        }

        public void BuildOtherError()
        {
            if (OtherErrorList.Count != 0)
            {
                string[] result = new string[OtherErrorList.Count];
                OtherErrorList.CopyTo( result );
                OtherError = String.Format(
                    "Other error(s) in the following line number(s) in your csv file:<br />{0}<br/>" +
                    "Details of the first few error(s):<br/>{1}",
                    String.Join( ", ", result ),
                    CreateOtherErrorDetails() );
            }
        }

        public void BuildCustomerNotFoundError()
        {
            if (CustomerNotFoundErrorList.Count != 0)
            {
                string[] result = new string[CustomerNotFoundErrorList.Count];
                CustomerNotFoundErrorList.CopyTo( result );
                CustomerNotFoundError = String.Format(
                    "Customers do not exist.<br/>" +
                    "Please verify Customers of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildHeaderError()
        {
            if (HeaderErrorList.Count != 0)
            {
                string[] result = new string[HeaderErrorList.Count];
                HeaderErrorList.CopyTo( result );
                HeaderError = String.Format(
                    "CSV header(s) are invalid.<br/>" +
                    "Please verify the following header(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildFieldError()
        {
            if (FieldErrorList.Count != 0)
            {
                string[] result = new string[FieldErrorList.Count];
                FieldErrorList.CopyTo( result );
                FieldError = String.Format(
                    "Some Customer fields are invalid.<br/>" +
                    "Please verify Customers of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }

        public void BuildShippingFieldError()
        {
            if (ShippingFieldErrorList.Count != 0)
            {
                string[] result = new string[ShippingFieldErrorList.Count];
                ShippingFieldErrorList.CopyTo( result );
                ShippingFieldError = String.Format(
                    "Some Shipping Address fields are invalid.<br/>" +
                    "Please verify Shipping Address of the following line number(s) in your csv file:<br />{0}<br/>",
                    String.Join( ", ", result ) );
            }
        }
    }
}
