﻿using System.Collections.Specialized;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;

namespace Vevo.Domain.Payments.PayPal
{
    public class PayPalRedirectPaymentMethod : RedirectPaymentMethod
    {
        private string EncodeSpecialXmlCharacters( string text )
        {
            text = text.Replace( "&", "&amp;" );
            text = text.Replace( "<", "&lt;" );
            text = text.Replace( ">", "&gt;" );
            text = text.Replace( "\"", "&quot;" );
            text = text.Replace( "'", "&apos;" );
            return text;
        }

        public override string GetPostedUrl() 
        {
            if (DataAccessContext.Configurations.GetBoolValue( "PaymentByPayPalEnvironment" ))
                return "https://www.sandbox.paypal.com/cgi-bin/webscr";
            else
                return "https://www.paypal.com/cgi-bin/webscr";
        }

        public override NameValueCollection CreatePostParameters(             
            CheckoutDetails checkoutDetails,
            Culture culture,
            Currency currency,
            Cart shoppingCart,
            string storefrontUrl,
            string orderID,
            decimal orderTotal,
            WholesaleStatus wholesaleStatus,
            string ipAddress )
        {
            NameValueCollection collection = new NameValueCollection();

            string returnURL = storefrontUrl + "Gateway/PayPalReturn.aspx?OrderID=" + orderID;
            string cancelURL = storefrontUrl;
            string notifyURL = storefrontUrl + "Gateway/PayPalNotification.aspx";

            collection.Add( "cmd", "_xclick" );
            collection.Add( "business", DataAccessContext.Configurations.GetValue( "PaymentByPayPalEmail" ) );
            collection.Add( "item_name", "Payment for " + EncodeSpecialXmlCharacters( DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) ) );
            collection.Add( "amount", Currency.ConvertPriceToUSFormat( orderTotal ) );
            collection.Add( "charset", "utf-8" );
            collection.Add( "no_shipping", "1" );
            collection.Add( "currency_code", DataAccessContext.Configurations.GetValue( "PaymentCurrency" ) );

            collection.Add( "address_override", "1" );
            collection.Add( "first_name", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.FirstName ) );
            collection.Add( "last_name", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.LastName ) );
            collection.Add( "address1", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.Address1 ) );
            collection.Add( "address2", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.Address1 ) );
            collection.Add( "city", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.City ) );
            collection.Add( "state", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.State ) );
            collection.Add( "zip", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.Zip) );
            collection.Add( "country", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.Country ) );

            collection.Add( "return", returnURL );
            collection.Add( "lc", EncodeSpecialXmlCharacters( checkoutDetails.ShippingAddress.Country ) );
            collection.Add( "cancel_return", cancelURL );
            collection.Add( "no_note", "1" );
            collection.Add( "notify_url", notifyURL );
            collection.Add( "invoice", orderID );

            return collection; 
        }
    }
}
