using System;
using Vevo.Domain;
using Vevo.QuickBooks.Qbxml70;
using Vevo.Domain.Orders;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for OrderAddResponseState
    /// </summary>
    public class OrderAddResponseState : QBWCState
    {
        private void ProcessRecieve( string response )
        {
            QBXMLMsgsRs qbXMLMsgsRs = CreateResponse( response );

            int successCount = 0;
            int failureCount = 0;

            for (int i = 0; i < qbXMLMsgsRs.Items.Length; i++)
            {
                SalesReceiptAddRsType result = (SalesReceiptAddRsType) qbXMLMsgsRs.Items[i];

                string orderID = string.Empty;
                if (result.statusCode.ToString() == "0")
                {
                    orderID = result.SalesReceiptRet.RefNumber.Replace( "OnlineID", "" );
                    Order order = DataAccessContext.OrderRepository.GetOne( orderID );
                    order.QBTxnID = result.SalesReceiptRet.TxnID;
                    order.IsQBExported = true;
                    DataAccessContext.OrderRepository.Save( order );
                    successCount++;
                }
                else
                {
                    CreateQBLogItem( result.statusCode, result.statusSeverity, result.statusMessage, QBExportType.OrderAdd );
                    failureCount++;
                }
            }
            CreateQBLogSummary( QBExportType.OrderAdd, successCount, failureCount );
        }

        public OrderAddResponseState( QBWCSession session )
            : base( session )
        {
        }

        public override int OnReceiveResponseXml( string responseXml )
        {
            ProcessRecieve( responseXml );
            // this is final state
            return 100;
        }
    }
}