﻿using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;

namespace Vevo.Reports.Exporters
{
    public class StockReportExporter
    {
        private string GenerateHeader()
        {
            return "\"Product ID\",\"Product Name\",\"SKU\",\"Product Option\",\"Stock\"";
        }

        private string GenerateDetail( DataRow row )
        {
            string ID,name, SKU, option,stock;
            ID = row["ProductID"].ToString();
            name = row["Name"].ToString();
            SKU = row["SKU"].ToString();
            option = row["Option"].ToString();
            stock = row["Stock"].ToString();
            return ID + "," + name + "," + SKU + "," + option + "," + stock;
        }

        private void WriteTableToCSVFile( DataTable table, string filePhysicalPathName ,string fullSortText)
        {
            DataRow[] rows = table.Select( "", fullSortText );
            if (rows.Length > 0)
            {
                StreamWriter writer = new StreamWriter( filePhysicalPathName, false );
                writer.WriteLine( GenerateHeader() );

                string details;
                foreach (DataRow row in rows)
                {
                    details = GenerateDetail( row );
                    writer.WriteLine( details );
                }
                writer.Dispose();
            }
        }

        public string ExportAllStockReportData(
            string filePhysicalPath,
            string fullSortText,
            string cultureID,
            out string message,
            out string message1,
            out string fileNameLinkText )
        {
            int totalItems;
            StockReportBuilder stockReportBuilder = new StockReportBuilder();
            DataTable table = stockReportBuilder.GetAllStockReport( cultureID, out totalItems );

            DateTime genDate = DateTime.Now;
            string filePathName = "Export/StockReport_All_" +
                genDate.Month + "_" + genDate.Day + "_" + genDate.Year + ".csv";

            string filePhysicalPathName = filePhysicalPath + filePathName;

            WriteTableToCSVFile( table, filePhysicalPathName, fullSortText );

            if (table.Rows.Count == 0)
            {
                message = "No data to export.";
                message1 = "";
                fileNameLinkText = "";
                return "";
            }
            else
            {
                message = "Export file is successfully create.";
                message1 = "Download ";
                fileNameLinkText = Path.GetFileName( filePathName );
                return "../DownloadFile.aspx?FilePath=" + "../" + filePathName;
            }
        }

        public string ExportLowStockReportData(
            string filePhysicalPath,
            string fullSortText,
            string cultureID,
            out string message,
            out string message1,
            out string fileNameLinkText )
        {
            int totalItems;
            StockReportBuilder stockReportBuilder = new StockReportBuilder();
            DataTable table = stockReportBuilder.GetLowStockReport( cultureID, out totalItems );

            DateTime genDate = DateTime.Now;
            string filePathName = "Export/StockReport_Low_" +
                genDate.Month + "_" + genDate.Day + "_" + genDate.Year + ".csv";

            string filePhysicalPathName = filePhysicalPath + filePathName;

            WriteTableToCSVFile( table, filePhysicalPathName, fullSortText );

            if (table.Rows.Count == 0)
            {
                message = "No data to export.";
                message1 = "";
                fileNameLinkText = "";
                return "";
            }
            else
            {
                message = "Export file is successfully create.";
                message1 = "Download ";
                fileNameLinkText = Path.GetFileName( filePathName );
                return "../DownloadFile.aspx?FilePath=" + "../" + filePathName;
            }
        }
    }
}
