using System;
using System.Data;
using System.Configuration;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Vevo.WebAppLib;
using Vevo.WebUI;
using Vevo.WebUI.Ajax;

namespace Vevo.WebUI.International
{
    /// <summary>
    /// Summary description for BaseLanguagePage
    /// </summary>
    public abstract class BaseLanguagePage : Vevo.WebUI.BaseControls.BasePage
    {
        private string ReadHtmlSource()
        {
            StringBuilder sb = new StringBuilder();
            StringWriter sw = new StringWriter( sb );
            HtmlTextWriter htmlWriter = new HtmlTextWriter( sw );
            base.Render( htmlWriter );

            return sb.ToString();
        }

        private PageTokenProcessor CreateTokenProcessor()
        {
            LanguageTokenRule rule = new LanguageTokenRule(
                AppRelativeVirtualPath.Trim( '~' ),
                CultureUtilities.StoreCultureID,
                CultureUtilities.BaseCultureID );

            PageTokenProcessor processor = new PageTokenProcessor( rule );

            return processor;
        }

        protected override void Render( HtmlTextWriter writer )
        {
            string source = ReadHtmlSource();

            ScriptManager scriptManager = AjaxUtilities.GetScriptManager( this );
            if (scriptManager == null ||
                !scriptManager.IsInAsyncPostBack)
            {
                StringBuilder output = CreateTokenProcessor().Process( source );

                writer.Write( output.ToString() );
            }
            else
            {
                AjaxPostbackData ajaxData;
                if (AjaxPostbackData.TryParse( source, out ajaxData ))
                {
                    foreach (AjaxPostbackBlock block in ajaxData.Blocks)
                    {
                        block.Body = CreateTokenProcessor().Process( block.Body ).ToString();
                    }
                    writer.Write( ajaxData.ToString() );
                }
                else
                {
                    writer.Write( source );
                }
            }
        }

        protected StorefrontEvents GetStorefrontEvents()
        {
            if (Master != null &&
                Master is BaseLanguageMasterPage)
                return ((BaseLanguageMasterPage) Page.Master).StorefrontEvents;
            else
                return StorefrontEvents.NullInstance;
        }

    }
}
