using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.SessionState;

using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.WebAppLib;
using Vevo.Domain.Stores;
using Vevo.Domain.Products;
using System.Collections;
using System.Web.Security;
using Vevo.Deluxe.Domain.CustomerReward;
using Vevo.Deluxe.Domain.Orders;

namespace Vevo.WebUI
{
    public static class StoreContext
    {
        #region Private

        private const string CheckoutSessionName = "Vevo_CheckoutSession";
        private const string WholesaleSessionName = "Vevo_WholesaleSession";
        private const string CultureSessionName = "Vevo_CultureSessionName";
        private const string ShoppingCartSessionName = "Vevo_ShopppingCartSession";
        private const string DisplayCurrencySessionName = "Vevo_DisplayCurrencySession";
        private const string RecentlyViewedSessionName = "Vevo_RecentlyViewedSession";
        private const string CompareListSessionName = "Vevo_CompareListSession";
        //private static string _cultureID = "0";
        //private static CheckoutDetails _checkoutDetails = new CheckoutDetails();
        //private static Cart _shoppingCart = new Cart();

        private static Customer _currentCustomer;

        private static HttpSessionState Session
        {
            get { return HttpContext.Current.Session; }
        }

        private static string DefaultCultureID
        {
            get
            {
                string defaultWebsite = "en-US";
                try
                {
                    defaultWebsite = DataAccessContext.Configurations.GetValueNoThrow( "DefaultWebsiteLanguage" );
                }
                catch { }

                Culture culture = DataAccessContext.CultureRepository.GetOne( DataAccessContext.CultureRepository.GetIDByName(
                   defaultWebsite ) );

                if (culture.IsNull)
                {
                    IList<Culture> list = DataAccessContext.CultureRepository.GetAll();
                    if (list.Count > 0)
                        return list[0].CultureID;
                }

                return culture.CultureID;
            }
        }

        private static string DefaultDisplayCurrencyCode
        {
            get
            {
                string defaultDisplayCurrencyCode = "USD";
                try
                {
                    defaultDisplayCurrencyCode = DataAccessContext.Configurations.GetValueNoThrow( "DefaultDisplayCurrencyCode" );
                }
                catch { }

                Currency currency = DataAccessContext.CurrencyRepository.GetOne( defaultDisplayCurrencyCode );

                if (currency.IsNull)
                {
                    IList<Currency> list = DataAccessContext.CurrencyRepository.GetByEnabled( BoolFilter.ShowTrue );
                    if (list.Count > 0)
                        return list[0].CurrencyCode;
                }

                return currency.CurrencyCode;
            }
        }

        #endregion


        #region Public Properties

        public static string StoreCultureID
        {
            get
            {
                return Culture.CultureID;
            }
            set
            {
                Culture = DataAccessContext.CultureRepository.GetOne( value );
            }
        }

        public static Store CurrentStore
        {
            get
            {
                return new StoreRetriever().GetStore();
            }
        }

        public static Culture Culture
        {
            get
            {
                if (Session[CultureSessionName] == null)
                    Session[CultureSessionName] = DataAccessContext.CultureRepository.GetOne( DefaultCultureID );

                return (Culture) Session[CultureSessionName];
            }
            private set
            {
                Session[CultureSessionName] = value;
            }
        }

        public static string StoreCurrencyCode
        {
            get
            {
                return Currency.CurrencyCode;
            }
            set
            {
                Currency = DataAccessContext.CurrencyRepository.GetOne( value );
            }
        }

        public static Currency Currency
        {
            get
            {
                if (Session[DisplayCurrencySessionName] == null)
                    Session[DisplayCurrencySessionName] = DataAccessContext.CurrencyRepository.GetOne( DefaultDisplayCurrencyCode );

                return (Currency) Session[DisplayCurrencySessionName];
            }
            private set
            {
                Session[DisplayCurrencySessionName] = value;
            }
        }

        public static string ConnectionStringsTagName
        {
            get
            {
                PaymentModuleSetup paymentModuleSetup = new PaymentModuleSetup();
                if (paymentModuleSetup.IsPaymentModuleIntegrated)
                {
                    return "MainConnection";
                }
                else
                    return "StoreConnection";
            }
        }

        public static Customer Customer
        {
            get
            {
                if ((HttpContext.Current.User.Identity.IsAuthenticated)
                    && (Roles.IsUserInRole( "Customers" )))
                {
                    string customerID = DataAccessContext.CustomerRepository.GetIDFromUserName(
                        HttpContext.Current.User.Identity.Name );

                    return DataAccessContext.CustomerRepository.GetOne( customerID );
                }
                else if ((HttpContext.Current.User.Identity.IsAuthenticated)
                    && (Roles.IsUserInRole( "Administrators" )))
                {
                    if (_currentCustomer != null)
                        return _currentCustomer;

                }
                return Customer.Null;
            }
            set
            {
                _currentCustomer = value;
            }
        }

        public static CheckoutDetails CheckoutDetails
        {
            get
            {
                if (HttpContext.Current.Session[CheckoutSessionName] == null)
                    HttpContext.Current.Session[CheckoutSessionName] = new CheckoutDetails();
                return (CheckoutDetails) HttpContext.Current.Session[CheckoutSessionName];
            }
        }

        public static Cart ShoppingCart
        {
            get
            {
                if (HttpContext.Current.Session[ShoppingCartSessionName] == null)
                    HttpContext.Current.Session[ShoppingCartSessionName] = new Cart();
                return (Cart) HttpContext.Current.Session[ShoppingCartSessionName];
            }
        }

        public static WishList WishList
        {
            get
            {
                string wishListID = DataAccessContext.WishListRepository.GetWishListIDFromCustomerID(
                    Customer.CustomerID );
                WishList wishList = DataAccessContext.WishListRepository.GetOne( wishListID );

                if (wishList.IsNull)
                {
                    Cart cart = new Cart();
                    cart = DataAccessContext.CartRepository.CreateWhole( cart );

                    wishList = new WishList();
                    wishList.CustomerID = Customer.CustomerID;
                    wishList.CartID = cart.CartID;
                    wishList = DataAccessContext.WishListRepository.Save( wishList );
                }

                return wishList;
            }
        }

        public static ArrayList ProductIDsCompareList
        {
            get
            {
                if (HttpContext.Current.Session[CompareListSessionName] == null)
                    HttpContext.Current.Session[CompareListSessionName] = new ArrayList();
                return (ArrayList) HttpContext.Current.Session[CompareListSessionName];
            }
        }

        public static WholesaleStatus WholesaleStatus
        {
            get
            {
                return Customer.GetWholesaleStatus();
            }
        }

        public static ArrayList RecentltViewedProduct
        {
            get
            {
                if (HttpContext.Current.Session[RecentlyViewedSessionName] == null)
                    HttpContext.Current.Session[RecentlyViewedSessionName] = new ArrayList();
                return (ArrayList) HttpContext.Current.Session[RecentlyViewedSessionName];
            }
        }

        #endregion


        #region Public Methods

        public static void ClearCheckoutSession()
        {
            HttpContext.Current.Session[CheckoutSessionName] = null;
            HttpContext.Current.Session[ShoppingCartSessionName] = null;
        }

        public static OrderAmount GetOrderAmount()
        {
            OrderCalculator orderCalculator = new OrderCalculator();

            OrderAmount orderAmount = orderCalculator.Calculate(
                CheckoutDetails,
                ShoppingCart.SeparateCartItemGroups(),
                Customer ).Add( CartItemPromotion.CalculatePromotionShippingAndTax(
                CheckoutDetails,
                ShoppingCart.SeparateCartItemGroups(),
                Customer ) );

            return CustomerRewardPoint.GetOrderAmountWithPointDiscount( orderAmount, CheckoutDetails );
        }

        public static OrderAmountPackage GetOrderAmountPackage()
        {
            OrderCalculator orderCalculator = new OrderCalculator();

            return orderCalculator.CalculatePackage(
                CheckoutDetails,
                ShoppingCart.GetAllCartItems(),
                Customer );
        }

        #endregion

    }
}
