using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using Vevo.Domain.Configurations;
using Vevo.Domain;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;

namespace Vevo.WebUI.Widget
{
    public class Widget
    {
        #region public const
        public static string DEFAULT_TYPE = "Default";
        public static string CUSTOM_TYPE = "Custom";
        #endregion

        #region private

        private  String[] _parameterValue = null;

        private string _code = String.Empty;
        private bool _enabled = false;
        private string _name;

        private const string _paramterCode = "<!--CODE--!>";

        private int CountStringOccurrences( string Text, string Pattern )
        {
            int count = 0;
            int i = 0;
            while ((i = Text.IndexOf( Pattern, i )) != -1)
            {
                i += Pattern.Length;
                count++;
            }
            return count;
        }

        private bool CheckCodeWithParameter( string Code, String[] Parameter )
        {
            try
            {
                int count = CountStringOccurrences( Code, _paramterCode );
                if (count == Parameter.Length) return true;
                else return false;
            }
            catch
            {
                return false;
            }
        }

        #endregion

        #region constructor
        // ---- Default Code ----
        public Widget( string Name,string Code, String[] Parameter )
        {
            if (CheckCodeWithParameter( Code, Parameter ) == true)
            {
                _code = Code;

                _parameterValue = new String[Parameter.Length];
                Parameter.CopyTo( _parameterValue, 0 );

                _name = Name;
            }
            else
            {
                throw new Exception( "Widget: Parameters don't match with code" );
            }
        }

        public Widget( string Name, string Code, string Parameter )
        {
            int count = CountStringOccurrences( Code, _paramterCode );
            _parameterValue = new String[count];
            for (int i = 0; i < count; i++)
            {
                _parameterValue[i] = Parameter;
            }
            _code = Code;
            _name = Name;
        }

        // ---- Custom Code ----
        public Widget( string Name, string CustomCode )
        {
            _code = CustomCode;
            _name = Name;
        }

        // ---- Load from Configuration ----
        public Widget( string Name,Store store )
        {
            _name = Name;
            LoadConfig( store );
        }

        #endregion

        #region public property
        public static string ParameterCode
        {
            get { return _paramterCode; }
        }

        public bool Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }

        public string Code
        {
            get
            {
                if (_parameterValue == null)
                {
                    return _code;
                }
                else
                {
                    string outCode = String.Empty;
                    string[] subCode = Regex.Split( _code, _paramterCode );
                    for (int i = 0; i < subCode.Length; i++)
                    {
                        outCode += subCode[i];
                        if (i!=subCode.Length-1) outCode += _parameterValue[i];
                    }
                    return outCode;
                }
            }
        }

        public string Name
        {
            get { return _name; }
        }

        public bool IsDefaultCode
        {
            get
            {
                if (_parameterValue != null) return true;
                else return false;
            }
        }

        public bool IsCustomCode
        {
            get
            {
                return !IsDefaultCode;
            }
        }

        public string CodeType
        {
            get
            {
                if (IsDefaultCode == true) return Widget.DEFAULT_TYPE;
                else return Widget.CUSTOM_TYPE;
            }
        }

        #endregion

        #region property for configuration
        public string ConfigNameIsEnabled
        {
            get
            {
                return "Widget" + _name + "IsEnabled";
            }
        }

        public string ConfigNameType
        {
            get
            {
                return "Widget" + _name + "Type";
            }
        }

        public string ConfigNameCode
        {
            get
            {
                return "Widget" + _name + "Code";
            }
        }

        public string ConfigNameParameterValue
        {
            get
            {
                return "Widget" + _name + "ParameterValue";
            }
        }

        public static string GetConfigNameIsEnabled(string Name)
        {
            return "Widget" + Name + "IsEnabled";
        }

        public static string GetConfigNameType(string Name)
        {
            return "Widget" + Name + "Type";
        }

        public static string GetConfigNameCode(string Name)
        {
            return "Widget" + Name + "Code";
        }

        public static string GetConfigNameParameterValue(string Name)
        {
            return "Widget" + Name + "ParameterValue";
        }

        #endregion

        #region public method

        public string GetParameterValue( int Index )
        {
            if (_parameterValue == null) return String.Empty;
            return _parameterValue[Index];
        }

        public string GetParameterValue()
        {
            if (_parameterValue == null) return String.Empty;
            string outString = String.Empty;
            for (int i = 0; i < _parameterValue.Length; i++)
            {
                outString += "," + _parameterValue[i];
            }
            return outString.Substring( 1 );
        }

        public void SaveConfig(Store store)
        {
            DataAccessContext.ConfigurationRepository.UpdateValue(
                DataAccessContext.Configurations[ConfigNameIsEnabled],
                this.Enabled.ToString(),store );
            DataAccessContext.ConfigurationRepository.UpdateValue(
                DataAccessContext.Configurations[ConfigNameType],
                this.CodeType, store );
            DataAccessContext.ConfigurationRepository.UpdateValue(
                DataAccessContext.Configurations[ConfigNameCode],
                this.Code, store );
            DataAccessContext.ConfigurationRepository.UpdateValue(
                DataAccessContext.Configurations[ConfigNameParameterValue],
                this.GetParameterValue(), store );            
        }

        public void LoadConfig(Store store)
        {
            try
            {
                _enabled = ConvertUtilities.ToBoolean( DataAccessContext.Configurations.GetValue( ConfigNameIsEnabled,store ) );
                _code = DataAccessContext.Configurations.GetValue( ConfigNameCode,store );
                string codeType = DataAccessContext.Configurations.GetValue( ConfigNameType,store );
                if (codeType == Widget.DEFAULT_TYPE)
                {
                    string valueText = DataAccessContext.Configurations.GetValue( ConfigNameParameterValue,store );
                    string[] values = valueText.Split( ',' );
                    _parameterValue = new string[values.Length];
                    for (int i = 0; i < values.Length; i++)
                    {
                        _parameterValue[i] = values[i];
                    }
                }
                else
                {
                    _parameterValue = null;
                }
            }
            catch
            {
                throw new Exception( "Widget: Cannot load configuration" );
            }
        }
        #endregion
    }
}
